/*******************************************************************************
 * University of Illinois/NCSA
 * Open Source License
 *
 * Copyright (c) 2010 , NCSA.  All rights reserved.
 *
 * Developed by:
 * Cyberenvironments and Technologies (CET)
 * http://cet.ncsa.illinois.edu/
 *
 * National Center for Supercomputing Applications (NCSA)
 * http://www.ncsa.illinois.edu/
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal with the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimers.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in the
 *   documentation and/or other materials provided with the distribution.
 * - Neither the names of CET, University of Illinois/NCSA, nor the names
 *   of its contributors may be used to endorse or promote products
 *   derived from this Software without specific prior written permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 *******************************************************************************/
package edu.uiuc.ncsa.cet.bean.gis;

import java.io.Serializable;

import edu.uiuc.ncsa.cet.bean.CETBean;

/**
 * An abstract class to represent gml:_Geometry object. Any Geospatial object
 * (e.g. GeoPoint, gml:LineString,gml:Point, gml:Polygon) will be subclass of
 * this abstract class. It will provide general interface to represent any
 * spatial object. This object described using namespace
 * "http://www.opengis.net/gml/_Geometry".
 * 
 * Ontologies for GML can be found at
 * "http://www.w3.org/2005/Incubator/geo/XGR-geo-20071023/W3C_XGR_Geo_files/geo_2007.owl"
 * 
 * @author kkotwani
 */
public abstract class GeoSpatialGeometryBean extends CETBean implements
		Serializable {

	public float getAltitude() {
		throw new UnsupportedOperationException();
	}

	public void setAltitude(float altitude) {
		throw new UnsupportedOperationException();
	}

	public float getLatitude() {
		throw new UnsupportedOperationException();
	}

	public void setLatitude(float latitude) {
		throw new UnsupportedOperationException();
	}

	public float getLongitude() {
		throw new UnsupportedOperationException();
	}

	public void setLongitude(float longitude) {
		throw new UnsupportedOperationException();
	}
}
