/*******************************************************************************
 * University of Illinois/NCSA
 * Open Source License
 *
 * Copyright (c) 2010 , NCSA.  All rights reserved.
 *
 * Developed by:
 * Cyberenvironments and Technologies (CET)
 * http://cet.ncsa.illinois.edu/
 *
 * National Center for Supercomputing Applications (NCSA)
 * http://www.ncsa.illinois.edu/
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal with the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimers.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimers in the
 *   documentation and/or other materials provided with the distribution.
 * - Neither the names of CET, University of Illinois/NCSA, nor the names
 *   of its contributors may be used to endorse or promote products
 *   derived from this Software without specific prior written permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS WITH THE SOFTWARE.
 *******************************************************************************/
package edu.uiuc.ncsa.cet.bean.gis;

import java.io.Serializable;

import edu.uiuc.ncsa.cet.bean.BlobBean;
import edu.uiuc.ncsa.cet.bean.CETBean;

/**
 * An implementation of dcTerms:Location class. Location object which can be
 * defined using mailing address or geoSpatial geometry (Geopoint, line etc.),
 * or a blob of kml or gml file. dcTerms:Location or Vcard:Location namespaces
 * can be used to specify rdf type of this object.
 * 
 * @author kkotwani
 * 
 */
public class LocationBean extends CETBean implements Serializable {

	private static final long serialVersionUID = -2449313358872952351L;

	/**
	 * Name of location
	 */
	private String name = "";

	/**
	 * Description or text-string defining location. e.g. "Red building near
	 * lake shore" or "1205 W Clark Urbana IL-61801" or
	 * "http://www.icsm.gov.au/icsm/cgna/index.html".
	 */
	private String description = "";

	/**
	 * Mailing Address for location using Vcard ontologies. Null if location
	 * defined using something else.
	 */
	private AddressBean address = null;

	/**
	 * GML Geometry object for defining GeoSpatial location. It Can be GeoPoint,
	 * Line or Polygon etc..
	 */
	private GeoSpatialGeometryBean geoSpatialGeomerty = null;

	/**
	 * Blob for defining or describing location. It can be picture of a building
	 * or a KML file defining Geo-spatial object.
	 */
	private BlobBean blob = null; // Initially allowing one blob per location

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public AddressBean getAddress() {
		return address;
	}

	public void setAddress(AddressBean address) {
		this.address = address;
	}

	public GeoSpatialGeometryBean getGeoSpatialGeomerty() {
		return geoSpatialGeomerty;
	}

	public void setGeoSpatialGeomerty(GeoSpatialGeometryBean geoSpatialGeomerty) {
		this.geoSpatialGeomerty = geoSpatialGeomerty;
	}

	public BlobBean getBlob() {
		return blob;
	}

	public void setBlob(BlobBean blob) {
		this.blob = blob;
	}

}
